!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                     !
!  This is Fortran source code of applying jump detector LCK          !
!  to the SAR image. Jump detector LCK is proposed in the paper       !
!  Kang, Y., and Qiu, P.,`Jump detection in blurred regression        !
!  surfaces'.                                                         !
!  Creator : Yicheng Kang                                             !
!  Date: April 9, 2013                                                !
!                                                                     !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

program jumpd_lck

  implicit none

  double precision :: z(0:600, 0:600), z1(0:600, 0:600), x, y, temp, &
       x1, y1, sigma, ker, r00, r20, bhat, chat, bb, ra, fhat1, fhat2, &
       gradperp, diff, thresh, stdz, num1, den1, num2, den2, zq

  integer :: i, j, n, i1, j1, k, itemp, edge1(0:600, 0:600)

  external :: extend, modify1, modify2, ker

  open(unit = 1, file = 'sar.dat', status = 'old')
  open(unit = 2, file = 'lck.out', status = 'unknown')

  ! Assign values to parameters.

  n = 249
  sigma = 17.485D0 
  zq = 3.090232D0
  thresh = 20D0              
  k = 4
  ra = dble(k)/dble(n)

  ! Start to read in data.

  do i = 0, n
     do j = 0, n

        read(1, *)  z(i, j)

     end do
  end do

  ! Extend image to avoid boundary problems.

  Call extend(n, k, z, z1)

  ! Calculate first derivatives                       

  r00 = 0D0
  r20 = 0D0

  do i = -k, k
     do j = -k, k

        if (i**2 + j**2 <= k**2) then

           temp = ker(dble(i)/dble(k), dble(j)/dble(k))
           r00 = r00 + temp
           r20 = r20 + (dble(i)/dble(n))**2 * temp

        end if

     end do
  end do

  do i = k, n + k
     do j = k, n + k

        bhat = 0D0
        chat = 0D0

        do i1 = i - k, i + k
           do j1 = j - k, j + k

              if ((i1 - i)**2 + (j1 - j)**2 <= k**2) then

                 bb = ker(dble(i1 - i)/dble(k), dble(j1 - j)/dble(k)) &
                      * z1(i1, j1)
                 bhat = bhat +  dble(i1 - i)/dble(n) * bb
                 chat = chat +  dble(j1 - j)/dble(n) * bb

              end if

           end do
        end do

        bhat = bhat/r20
        chat = chat/r20

        ! To estimate the line which is perpendicular to gradient.

        x = dble(i - k)/dble(n)
        y = dble(j - k)/dble(n)
        fhat1 = 0D0
        fhat2 = 0D0
        num1 = 0D0
        den1 = 0D0
        num2 = 0D0
        den2 = 0D0

        do i1 = i - k, i + k
           do j1 = j - k, j + k

              x1 = dble(i1 - k)/dble(n)
              y1 = dble(j1 - k)/dble(n)

              if ((i1 - i)**2 + (j1 - j)**2 <= k**2) then

                 gradperp = bhat * (x1 - x) + chat * (y1 - y)
                 bb = ker((x1 - x)/ra, (y1 - y)/ra)

                 ! Start to fit two local constants.

                 if (gradperp >= 0D0) then

                    num1 = num1 + bb**2
                    den1 = den1 + bb
                    fhat1 = fhat1 + z1(i1, j1) * bb

                 else

                    num2 = num2 + bb**2
                    den2 = den2 + bb
                    fhat2 = fhat2 + z1(i1, j1) * bb

                 end if

              end if

           end do
        end do

        if (abs(den1) > 0D0) then

           fhat1 = fhat1/den1

        else

           fhat1 = z1(i, j)

        end if

        if (abs(den2) > 0D0) then

           fhat2 = fhat2/den2

        else

           fhat2 = z1(i, j)

        end if

        diff = fhat1 - fhat2
        stdz = sqrt(num1/(den1**2) + num2/(den2**2)) * sigma

        if (abs(diff/stdz) >= zq) then
        !if (abs(diff) >= thresh) then

           edge1(i, j) = 1

        else

           edge1(i, j) = 0

        end if

     end do
  end do

  itemp = 2 * k + 1

!  call modify1(n, itemp, k, z1, edge1)
  call modify2(n, itemp, k, edge1)

  do i = 0, n
     do j = 0, n

        write(2, '(I4)'), edge1(i + k, j + k)

     end do
  end do

end program jumpd_lck
